@@ -52,9 +52,34 @@ angular.module('syncthing.core')
         $scope.scanProgress = {};
         $scope.themes = [];
         $scope.globalChangeEvents = {};
+        $scope.metricRates = false;
+        $scope.folderPathErrors = {};
+
+        try {
+            $scope.metricRates = (window.localStorage["metricRates"] == "true");
+        } catch (exception) { }
+
+        $scope.folderDefaults = {
+            selectedDevices: {},
+            type: "readwrite",
+            rescanIntervalS: 60,
+            minDiskFree: {value: 1, unit: "%"},
+            maxConflicts: 10,
+            fsync: true,
+            order: "random",
+            fileVersioningSelector: "none",
+            trashcanClean: 0,
+            simpleKeep: 5,
+            staggeredMaxAge: 365,
+            staggeredCleanInterval: 3600,
+            staggeredVersionsPath: "",
+            externalCommand: "",
+            autoNormalize: true
+        };
 
         $scope.localStateTotal = {
             bytes: 0,
+            directories: 0,
             files: 0
         };
 
@@ -424,12 +449,14 @@ angular.module('syncthing.core')
         function recalcLocalStateTotal () {
             $scope.localStateTotal = {
                 bytes: 0,
+                directories: 0,
                 files: 0
             };
 
             for (var f in $scope.model) {
                $scope.localStateTotal.bytes += $scope.model[f].localBytes;
                $scope.localStateTotal.files += $scope.model[f].localFiles;
+               $scope.localStateTotal.directories += $scope.model[f].localDirectories;
             }
         }
 
@@ -722,6 +749,22 @@ angular.module('syncthing.core')
             return Math.floor(pct);
         };
 
+        $scope.syncRemaining = function (folder) {
+            // Remaining sync bytes
+            if (typeof $scope.model[folder] === 'undefined') {
+                return 0;
+            }
+            if ($scope.model[folder].globalBytes === 0) {
+                return 0;
+            }
+
+            var bytes = $scope.model[folder].globalBytes - $scope.model[folder].inSyncBytes;
+            if (isNaN(bytes) || bytes < 0) {
+                return 0;
+            }
+            return bytes;
+        };
+
         $scope.scanPercentage = function (folder) {
             if (!$scope.scanProgress[folder]) {
                 return undefined;
@@ -983,17 +1026,27 @@ angular.module('syncthing.core')
             }
         };
 
+        $scope.showDiscoveryFailures = function () {
+            $('#discovery-failures').modal();
+        };
+
         $scope.editSettings = function () {
             // Make a working copy
             $scope.tmpOptions = angular.copy($scope.config.options);
             $scope.tmpOptions.urEnabled = ($scope.tmpOptions.urAccepted > 0);
             $scope.tmpOptions.deviceName = $scope.thisDevice().name;
-            $scope.tmpOptions.autoUpgradeEnabled = ($scope.tmpOptions.autoUpgradeIntervalH > 0);
+            $scope.tmpOptions.upgrades = "none";
+            if ($scope.tmpOptions.autoUpgradeIntervalH > 0) {
+                $scope.tmpOptions.upgrades = "stable";
+            }
+            if ($scope.tmpOptions.upgradeToPreReleases) {
+                $scope.tmpOptions.upgrades = "candidate";
+            }
             $scope.tmpGUI = angular.copy($scope.config.gui);
             $('#settings').modal();
         };
 
-        $scope.saveConfig = function () {
+        $scope.saveConfig = function (cb) {
             var cfg = JSON.stringify($scope.config);
             var opts = {
                 headers: {
@@ -1003,6 +1056,9 @@ angular.module('syncthing.core')
             $http.post(urlbase + '/system/config', cfg, opts).success(function () {
                 $http.get(urlbase + '/system/config/insync').success(function (data) {
                     $scope.configInSync = data.configInSync;
+                    if (cb) {
+                        cb();
+                    }
                 });
             }).error($scope.emitHTTPError);
         };
@@ -1012,6 +1068,20 @@ angular.module('syncthing.core')
             var changed = !angular.equals($scope.config.options, $scope.tmpOptions) || !angular.equals($scope.config.gui, $scope.tmpGUI);
             var themeChanged = $scope.config.gui.theme !== $scope.tmpGUI.theme;
             if (changed) {
+                // Check if auto-upgrade has been enabled or disabled. This
+                // also has an effect on usage reporting, so do the check
+                // for that later.
+                if ($scope.tmpOptions.upgrades == "candidate") {
+                    $scope.tmpOptions.autoUpgradeIntervalH = $scope.tmpOptions.autoUpgradeIntervalH || 12;
+                    $scope.tmpOptions.upgradeToPreReleases = true;
+                    $scope.tmpOptions.urEnabled = true;
+                } else if ($scope.tmpOptions.upgrades == "stable") {
+                    $scope.tmpOptions.autoUpgradeIntervalH = $scope.tmpOptions.autoUpgradeIntervalH || 12;
+                    $scope.tmpOptions.upgradeToPreReleases = false;
+                } else {
+                    $scope.tmpOptions.autoUpgradeIntervalH = 0;
+                }
+
                 // Check if usage reporting has been enabled or disabled
                 if ($scope.tmpOptions.urEnabled && $scope.tmpOptions.urAccepted <= 0) {
                     $scope.tmpOptions.urAccepted = 1000;
@@ -1019,13 +1089,6 @@ angular.module('syncthing.core')
                     $scope.tmpOptions.urAccepted = -1;
                 }
 
-                // Check if auto-upgrade has been enabled or disabled
-                if ($scope.tmpOptions.autoUpgradeEnabled) {
-                    $scope.tmpOptions.autoUpgradeIntervalH = $scope.tmpOptions.autoUpgradeIntervalH || 12;
-                } else {
-                    $scope.tmpOptions.autoUpgradeIntervalH = 0;
-                }
-
                 // Check if protocol will need to be changed on restart
                 if ($scope.config.gui.useTLS !== $scope.tmpGUI.useTLS) {
                     $scope.protocolChanged = true;
@@ -1042,14 +1105,14 @@ angular.module('syncthing.core')
                     });
                 });
 
-                $scope.saveConfig();
+                $scope.saveConfig(function () {
+                    if (themeChanged) {
+                        document.location.reload(true);
+                    }
+                });
             }
 
             $('#settings').modal("hide");
-
-            if (themeChanged) {
-              document.location.reload(true);
-            }
         };
 
         $scope.saveAdvanced = function () {
@@ -1306,6 +1369,13 @@ angular.module('syncthing.core')
             $('#globalChanges').modal();
         };
 
+        $scope.editFolderModal = function () {
+            $scope.folderPathErrors = {};
+            $scope.folderEditor.$setPristine();
+            $('#editIgnores textarea').val("");
+            $('#editFolder').modal();
+        };
+
         $scope.editFolder = function (folderCfg) {
             $scope.currentFolder = angular.copy(folderCfg);
             if ($scope.currentFolder.path.slice(-1) === $scope.system.pathSeparator) {
@@ -1350,64 +1420,30 @@ angular.module('syncthing.core')
             $scope.currentFolder.externalCommand = $scope.currentFolder.externalCommand || "";
 
             $scope.editingExisting = true;
-            $scope.folderEditor.$setPristine();
-            $('#editFolder').modal();
+            $scope.editFolderModal();
         };
 
         $scope.addFolder = function () {
-            $scope.currentFolder = {
-                selectedDevices: {},
-                type: "readwrite",
-                rescanIntervalS: 60,
-                minDiskFreePct: 1,
-                maxConflicts: 10,
-                fsync: true,
-                order: "random",
-                fileVersioningSelector: "none",
-                trashcanClean: 0,
-                simpleKeep: 5,
-                staggeredMaxAge: 365,
-                staggeredCleanInterval: 3600,
-                staggeredVersionsPath: "",
-                externalCommand: "",
-                autoNormalize: true
-            };
-            $scope.editingExisting = false;
-            $scope.folderEditor.$setPristine();
             $http.get(urlbase + '/svc/random/string?length=10').success(function (data) {
+                $scope.currentFolder = angular.copy($scope.folderDefaults);
                 $scope.currentFolder.id = (data.random.substr(0, 5) + '-' + data.random.substr(5, 5)).toLowerCase();
-                $('#editFolder').modal();
+                $scope.editingExisting = false;
+                $scope.editFolderModal();
             });
         };
 
         $scope.addFolderAndShare = function (folder, folderLabel, device) {
             $scope.dismissFolderRejection(folder, device);
-            $scope.currentFolder = {
-                id: folder,
-                label: folderLabel,
-                selectedDevices: {},
-                rescanIntervalS: 60,
-                minDiskFreePct: 1,
-                maxConflicts: 10,
-                fsync: true,
-                order: "random",
-                fileVersioningSelector: "none",
-                trashcanClean: 0,
-                simpleKeep: 5,
-                staggeredMaxAge: 365,
-                staggeredCleanInterval: 3600,
-                staggeredVersionsPath: "",
-                externalCommand: "",
-                autoNormalize: true,
-                viewFlags: {
-                    importFromOtherDevice: true
-                }
+            $scope.currentFolder = angular.copy($scope.folderDefaults);
+            $scope.currentFolder.id = folder;
+            $scope.currentFolder.label = folderLabel;
+            $scope.currentFolder.viewFlags = {
+                importFromOtherDevice: true
             };
             $scope.currentFolder.selectedDevices[device] = true;
 
             $scope.editingExisting = false;
-            $scope.folderEditor.$setPristine();
-            $('#editFolder').modal();
+            $scope.editFolderModal();
         };
 
         $scope.shareFolderWithDevice = function (folder, device) {
@@ -1478,16 +1514,33 @@ angular.module('syncthing.core')
                 delete folderCfg.versioning;
             }
 
+            var ignores = $('#editIgnores textarea').val().trim();
+            if (!$scope.editingExisting && ignores) {
+                folderCfg.paused = true;
+            };
+
             $scope.folders[folderCfg.id] = folderCfg;
             $scope.config.folders = folderList($scope.folders);
 
-            $scope.saveConfig();
+            $scope.saveConfig(function () {
+                if (!$scope.editingExisting && ignores) {
+                    $scope.saveIgnores(function () {
+                        $scope.setFolderPause(folderCfg.id, false);
+                    });
+                }
+            });
         };
 
         $scope.dismissFolderRejection = function (folder, device) {
             delete $scope.folderRejections[folder + "-" + device];
         };
 
+        $scope.ignoreRejectedFolder = function (folder, device) {
+            $scope.config.ignoredFolders.push(folder);
+            $scope.saveConfig();
+            $scope.dismissFolderRejection(folder, device);
+        };
+
         $scope.sharesFolder = function (folderCfg) {
             var names = [];
             folderCfg.devices.forEach(function (device) {
@@ -1555,13 +1608,27 @@ angular.module('syncthing.core')
                 });
         };
 
-        $scope.saveIgnores = function () {
-            if (!$scope.editingExisting) {
+        $scope.editIgnoresOnAddingFolder = function () {
+            if ($scope.editingExisting) {
                 return;
             }
 
+            if ($scope.currentFolder.path.endsWith($scope.system.pathSeparator)) {
+                $scope.currentFolder.path = $scope.currentFolder.path.slice(0, -1);
+            };
+            $('#editIgnores').modal().one('shown.bs.modal', function () {
+                textArea.focus();
+            });
+        };
+
+
+        $scope.saveIgnores = function (cb) {
             $http.post(urlbase + '/db/ignores?folder=' + encodeURIComponent($scope.currentFolder.id), {
                 ignore: $('#editIgnores textarea').val().split('\n')
+            }).success(function () {
+                if (cb) {
+                    cb();
+                }
             });
         };
 
@@ -1596,6 +1663,10 @@ angular.module('syncthing.core')
 
         $scope.showFailed = function (folder) {
             $scope.failedCurrent = $scope.failed[folder];
+            $scope.failedFolderPath = $scope.folders[folder].path;
+            if ($scope.failedFolderPath[$scope.failedFolderPath.length - 1] !== $scope.system.pathSeparator) {
+                $scope.failedFolderPath += $scope.system.pathSeparator;
+            }
             $('#failed').modal().on('hidden.bs.modal', function () {
                 $scope.failedCurrent = undefined;
             });
@@ -1632,6 +1703,29 @@ angular.module('syncthing.core')
             $http.post(urlbase + "/db/scan?folder=" + encodeURIComponent(folder));
         };
 
+        $scope.setAllFoldersPause = function(pause) {
+            var folderListCache = $scope.folderList();
+
+            for (var i = 0; i < folderListCache.length; i++) {
+                folderListCache[i].paused = pause;
+            }
+
+            $scope.config.folders = folderList(folderListCache);
+            $scope.saveConfig();
+        };
+
+        $scope.isAtleastOneFolderPausedStateSetTo = function(pause) {
+            var folderListCache = $scope.folderList();
+
+            for (var i = 0; i < folderListCache.length; i++) {
+                if (folderListCache[i].paused == pause) {
+                    return true;
+                }
+            }
+
+            return false;
+        };
+
         $scope.bumpFile = function (folder, file) {
             var url = urlbase + "/db/prio?folder=" + encodeURIComponent(folder) + "&file=" + encodeURIComponent(file);
             // In order to get the right view of data in the response.
@@ -1686,6 +1780,9 @@ angular.module('syncthing.core')
             if (typeof value === 'boolean') {
                 return 'checkbox';
             }
+            if (value instanceof Array) {
+                return 'list';
+            }
             if (typeof value === 'object') {
                 return 'skip';
             }
@@ -1699,7 +1796,6 @@ angular.module('syncthing.core')
         };
 
         $scope.modalLoaded = function () {
-
             // once all modal elements have been processed
             if ($('modal').length === 0) {
 
@@ -1708,4 +1804,10 @@ angular.module('syncthing.core')
             }
         }
 
+        $scope.toggleUnits = function () {
+            $scope.metricRates = !$scope.metricRates;
+            try {
+                window.localStorage["metricRates"] = $scope.metricRates;
+            } catch (exception) { }
+        }
     });
