@@ -2,7 +2,7 @@
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this file,
-// You can obtain one at http://mozilla.org/MPL/2.0/.
+// You can obtain one at https://mozilla.org/MPL/2.0/.
 
 package config
 
@@ -54,9 +54,8 @@ func TestDefaultValues(t *testing.T) {
 		KeepTemporariesH:        24,
 		CacheIgnoredFiles:       false,
 		ProgressUpdateIntervalS: 5,
-		SymlinksEnabled:         true,
 		LimitBandwidthInLan:     false,
-		MinHomeDiskFreePct:      1,
+		MinHomeDiskFree:         Size{1, "%"},
 		URURL:                   "https://data.syncthing.net/newdata",
 		URInitialDelayS:         1800,
 		URPostInsecurely:        false,
@@ -65,6 +64,15 @@ func TestDefaultValues(t *testing.T) {
 		OverwriteRemoteDevNames: false,
 		TempIndexMinBlocks:      10,
 		UnackedNotificationIDs:  []string{},
+		WeakHashSelectionMethod: WeakHashAuto,
+		StunKeepaliveS:          24,
+		StunServers:             []string{"default"},
+		DefaultKCPEnabled:       false,
+		KCPCongestionControl:    true,
+		KCPReceiveWindowSize:    128,
+		KCPSendWindowSize:       128,
+		KCPUpdateIntervalMs:     25,
+		KCPFastResend:           false,
 	}
 
 	cfg := New(device1)
@@ -102,12 +110,13 @@ func TestDeviceConfig(t *testing.T) {
 				Pullers:         0,
 				Hashers:         0,
 				AutoNormalize:   true,
-				MinDiskFreePct:  1,
+				MinDiskFree:     Size{1, "%"},
 				MaxConflicts:    -1,
 				Fsync:           true,
 				Versioning: VersioningConfiguration{
 					Params: map[string]string{},
 				},
+				WeakHashThresholdPct: 25,
 			},
 		}
 
@@ -124,16 +133,18 @@ func TestDeviceConfig(t *testing.T) {
 
 		expectedDevices := []DeviceConfiguration{
 			{
-				DeviceID:    device1,
-				Name:        "node one",
-				Addresses:   []string{"tcp://a"},
-				Compression: protocol.CompressMetadata,
+				DeviceID:        device1,
+				Name:            "node one",
+				Addresses:       []string{"tcp://a"},
+				Compression:     protocol.CompressMetadata,
+				AllowedNetworks: []string{},
 			},
 			{
-				DeviceID:    device4,
-				Name:        "node two",
-				Addresses:   []string{"tcp://b"},
-				Compression: protocol.CompressMetadata,
+				DeviceID:        device4,
+				Name:            "node two",
+				Addresses:       []string{"tcp://b"},
+				Compression:     protocol.CompressMetadata,
+				AllowedNetworks: []string{},
 			},
 		}
 		expectedDeviceIDs := []protocol.DeviceID{device1, device4}
@@ -189,9 +200,8 @@ func TestOverriddenValues(t *testing.T) {
 		KeepTemporariesH:        48,
 		CacheIgnoredFiles:       true,
 		ProgressUpdateIntervalS: 10,
-		SymlinksEnabled:         false,
 		LimitBandwidthInLan:     true,
-		MinHomeDiskFreePct:      5.2,
+		MinHomeDiskFree:         Size{5.2, "%"},
 		URURL:                   "https://localhost/newdata",
 		URInitialDelayS:         800,
 		URPostInsecurely:        true,
@@ -199,9 +209,21 @@ func TestOverriddenValues(t *testing.T) {
 		AlwaysLocalNets:         []string{},
 		OverwriteRemoteDevNames: true,
 		TempIndexMinBlocks:      100,
-		UnackedNotificationIDs:  []string{},
+		UnackedNotificationIDs: []string{
+			"channelNotification", // added in 17->18 migration
+		},
+		WeakHashSelectionMethod: WeakHashNever,
+		StunKeepaliveS:          10,
+		StunServers:             []string{"a.stun.com", "b.stun.com"},
+		DefaultKCPEnabled:       true,
+		KCPCongestionControl:    false,
+		KCPReceiveWindowSize:    1280,
+		KCPSendWindowSize:       1280,
+		KCPUpdateIntervalMs:     1000,
+		KCPFastResend:           true,
 	}
 
+	os.Unsetenv("STNOUPGRADE")
 	cfg, err := Load("testdata/overridenvalues.xml", device1)
 	if err != nil {
 		t.Error(err)
@@ -216,22 +238,26 @@ func TestDeviceAddressesDynamic(t *testing.T) {
 	name, _ := os.Hostname()
 	expected := map[protocol.DeviceID]DeviceConfiguration{
 		device1: {
-			DeviceID:  device1,
-			Addresses: []string{"dynamic"},
+			DeviceID:        device1,
+			Addresses:       []string{"dynamic"},
+			AllowedNetworks: []string{},
 		},
 		device2: {
-			DeviceID:  device2,
-			Addresses: []string{"dynamic"},
+			DeviceID:        device2,
+			Addresses:       []string{"dynamic"},
+			AllowedNetworks: []string{},
 		},
 		device3: {
-			DeviceID:  device3,
-			Addresses: []string{"dynamic"},
+			DeviceID:        device3,
+			Addresses:       []string{"dynamic"},
+			AllowedNetworks: []string{},
 		},
 		device4: {
-			DeviceID:    device4,
-			Name:        name, // Set when auto created
-			Addresses:   []string{"dynamic"},
-			Compression: protocol.CompressMetadata,
+			DeviceID:        device4,
+			Name:            name, // Set when auto created
+			Addresses:       []string{"dynamic"},
+			Compression:     protocol.CompressMetadata,
+			AllowedNetworks: []string{},
 		},
 	}
 
@@ -250,25 +276,29 @@ func TestDeviceCompression(t *testing.T) {
 	name, _ := os.Hostname()
 	expected := map[protocol.DeviceID]DeviceConfiguration{
 		device1: {
-			DeviceID:    device1,
-			Addresses:   []string{"dynamic"},
-			Compression: protocol.CompressMetadata,
+			DeviceID:        device1,
+			Addresses:       []string{"dynamic"},
+			Compression:     protocol.CompressMetadata,
+			AllowedNetworks: []string{},
 		},
 		device2: {
-			DeviceID:    device2,
-			Addresses:   []string{"dynamic"},
-			Compression: protocol.CompressMetadata,
+			DeviceID:        device2,
+			Addresses:       []string{"dynamic"},
+			Compression:     protocol.CompressMetadata,
+			AllowedNetworks: []string{},
 		},
 		device3: {
-			DeviceID:    device3,
-			Addresses:   []string{"dynamic"},
-			Compression: protocol.CompressNever,
+			DeviceID:        device3,
+			Addresses:       []string{"dynamic"},
+			Compression:     protocol.CompressNever,
+			AllowedNetworks: []string{},
 		},
 		device4: {
-			DeviceID:    device4,
-			Name:        name, // Set when auto created
-			Addresses:   []string{"dynamic"},
-			Compression: protocol.CompressMetadata,
+			DeviceID:        device4,
+			Name:            name, // Set when auto created
+			Addresses:       []string{"dynamic"},
+			Compression:     protocol.CompressMetadata,
+			AllowedNetworks: []string{},
 		},
 	}
 
@@ -287,22 +317,26 @@ func TestDeviceAddressesStatic(t *testing.T) {
 	name, _ := os.Hostname()
 	expected := map[protocol.DeviceID]DeviceConfiguration{
 		device1: {
-			DeviceID:  device1,
-			Addresses: []string{"tcp://192.0.2.1", "tcp://192.0.2.2"},
+			DeviceID:        device1,
+			Addresses:       []string{"tcp://192.0.2.1", "tcp://192.0.2.2"},
+			AllowedNetworks: []string{},
 		},
 		device2: {
-			DeviceID:  device2,
-			Addresses: []string{"tcp://192.0.2.3:6070", "tcp://[2001:db8::42]:4242"},
+			DeviceID:        device2,
+			Addresses:       []string{"tcp://192.0.2.3:6070", "tcp://[2001:db8::42]:4242"},
+			AllowedNetworks: []string{},
 		},
 		device3: {
-			DeviceID:  device3,
-			Addresses: []string{"tcp://[2001:db8::44]:4444", "tcp://192.0.2.4:6090"},
+			DeviceID:        device3,
+			Addresses:       []string{"tcp://[2001:db8::44]:4444", "tcp://192.0.2.4:6090"},
+			AllowedNetworks: []string{},
 		},
 		device4: {
-			DeviceID:    device4,
-			Name:        name, // Set when auto created
-			Addresses:   []string{"dynamic"},
-			Compression: protocol.CompressMetadata,
+			DeviceID:        device4,
+			Name:            name, // Set when auto created
+			Addresses:       []string{"dynamic"},
+			Compression:     protocol.CompressMetadata,
+			AllowedNetworks: []string{},
 		},
 	}
 
