@@ -2,17 +2,23 @@
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this file,
-// You can obtain one at http://mozilla.org/MPL/2.0/.
+// You can obtain one at https://mozilla.org/MPL/2.0/.
 
 package model
 
-import "time"
+import (
+	"context"
+	"time"
+)
 
 type folder struct {
 	stateTracker
-	scan  folderScanner
-	model *Model
-	stop  chan struct{}
+
+	scan                folderScanner
+	model               *Model
+	ctx                 context.Context
+	cancel              context.CancelFunc
+	initialScanFinished chan struct{}
 }
 
 func (f *folder) IndexUpdated() {
@@ -23,10 +29,12 @@ func (f *folder) DelayScan(next time.Duration) {
 }
 
 func (f *folder) Scan(subdirs []string) error {
+	<-f.initialScanFinished
 	return f.scan.Scan(subdirs)
 }
+
 func (f *folder) Stop() {
-	close(f.stop)
+	f.cancel()
 }
 
 func (f *folder) Jobs() ([]string, []string) {
@@ -35,13 +43,8 @@ func (f *folder) Jobs() ([]string, []string) {
 
 func (f *folder) BringToFront(string) {}
 
-func (f *folder) scanSubdirsIfHealthy(subDirs []string) error {
-	if err := f.model.CheckFolderHealth(f.folderID); err != nil {
-		l.Infoln("Skipping folder", f.folderID, "scan due to folder error:", err)
-		return err
-	}
-	l.Debugln(f, "Scanning subdirectories")
-	if err := f.model.internalScanFolderSubdirs(f.folderID, subDirs); err != nil {
+func (f *folder) scanSubdirs(subDirs []string) error {
+	if err := f.model.internalScanFolderSubdirs(f.ctx, f.folderID, subDirs); err != nil {
 		// Potentially sets the error twice, once in the scanner just
 		// by doing a check, and once here, if the error returned is
 		// the same one as returned by CheckFolderHealth, though
